@@ -1,4 +1,4 @@
-/* $Id$ */
+/* vi:set et ai sw=2 sts=2 ts=2: */
 /*-
  * Copyright (c) 2005-2006 Benedikt Meurer <benny@xfce.org>
  * Copyright (c) 2009 Jannis Pohlmann <jannis@xfce.org>
@@ -108,6 +108,7 @@ static void           thunar_location_buttons_scroll_left               (GtkWidg
 static void           thunar_location_buttons_scroll_right              (GtkWidget                  *button,
                                                                          ThunarLocationButtons      *buttons);
 static void           thunar_location_buttons_clicked                   (ThunarLocationButton       *button,
+                                                                         gboolean                    open_in_tab,
                                                                          ThunarLocationButtons      *buttons);
 static void           thunar_location_buttons_context_menu              (ThunarLocationButton       *button,
                                                                          GdkEventButton             *event,
@@ -122,6 +123,8 @@ static void           thunar_location_buttons_action_empty_trash        (GtkActi
                                                                          ThunarLocationButtons      *buttons);
 static void           thunar_location_buttons_action_open               (GtkAction                  *action,
                                                                          ThunarLocationButtons      *buttons);
+static void           thunar_location_buttons_action_open_in_new_tab    (GtkAction                  *action,
+                                                                         ThunarLocationButtons      *buttons);
 static void           thunar_location_buttons_action_open_in_new_window (GtkAction                  *action,
                                                                          ThunarLocationButtons      *buttons);
 static void           thunar_location_buttons_action_paste_into_folder  (GtkAction                  *action,
@@ -161,16 +164,21 @@ struct _ThunarLocationButtons
 
 
 
+static GQuark thunar_file_quark = 0;
+
+
+
 static const GtkActionEntry action_entries[] =
 {
   { "location-buttons-down-folder", NULL, "Down Folder", "<alt>Down", NULL, G_CALLBACK (thunar_location_buttons_action_down_folder), },
-  { "location-buttons-context-menu", NULL, "Context Menu", NULL, NULL, NULL, },
-  { "location-buttons-open", GTK_STOCK_OPEN, N_("_Open"), NULL, NULL, G_CALLBACK (thunar_location_buttons_action_open), },
-  { "location-buttons-open-in-new-window", NULL, N_("Open in New Window"), NULL, NULL, G_CALLBACK (thunar_location_buttons_action_open_in_new_window), },
-  { "location-buttons-create-folder", NULL, N_("Create _Folder..."), NULL, NULL, G_CALLBACK (thunar_location_buttons_action_create_folder), },
-  { "location-buttons-empty-trash", NULL, N_("_Empty Trash"), NULL, N_("Delete all files and folders in the Trash"), G_CALLBACK (thunar_location_buttons_action_empty_trash), },
-  { "location-buttons-paste-into-folder", GTK_STOCK_PASTE, N_("Paste Into Folder"), NULL, NULL, G_CALLBACK (thunar_location_buttons_action_paste_into_folder), },
-  { "location-buttons-properties", GTK_STOCK_PROPERTIES, N_("_Properties..."), NULL, NULL, G_CALLBACK (thunar_location_buttons_action_properties), },
+  { "location-buttons-context-menu", NULL, "Context Menu", NULL, "", NULL, },
+  { "location-buttons-open", GTK_STOCK_OPEN, N_("_Open"), "", NULL, G_CALLBACK (thunar_location_buttons_action_open), },
+  { "location-buttons-open-in-new-tab", NULL, N_("Open in New Tab"), "", NULL, G_CALLBACK (thunar_location_buttons_action_open_in_new_tab), },
+  { "location-buttons-open-in-new-window", NULL, N_("Open in New Window"), "", NULL, G_CALLBACK (thunar_location_buttons_action_open_in_new_window), },
+  { "location-buttons-create-folder", "folder-new", N_("Create _Folder..."), "", NULL, G_CALLBACK (thunar_location_buttons_action_create_folder), },
+  { "location-buttons-empty-trash", NULL, N_("_Empty Trash"), "", N_("Delete all files and folders in the Trash"), G_CALLBACK (thunar_location_buttons_action_empty_trash), },
+  { "location-buttons-paste-into-folder", GTK_STOCK_PASTE, N_("Paste Into Folder"), "", NULL, G_CALLBACK (thunar_location_buttons_action_paste_into_folder), },
+  { "location-buttons-properties", GTK_STOCK_PROPERTIES, N_("_Properties..."), "", NULL, G_CALLBACK (thunar_location_buttons_action_properties), },
 };
 
 
@@ -224,6 +232,8 @@ thunar_location_buttons_class_init (ThunarLocationButtonsClass *klass)
                                                              _("The amount of space between the path buttons"),
                                                              0, G_MAXINT, 3,
                                                              G_PARAM_READABLE));
+
+  thunar_file_quark = g_quark_from_static_string ("button-thunar-file");
 }
 
 
@@ -1105,6 +1115,7 @@ thunar_location_buttons_scroll_right (GtkWidget             *button,
 
 static void
 thunar_location_buttons_clicked (ThunarLocationButton  *button,
+                                 gboolean               open_in_tab,
                                  ThunarLocationButtons *buttons)
 {
   ThunarFile *directory;
@@ -1116,6 +1127,13 @@ thunar_location_buttons_clicked (ThunarLocationButton  *button,
   /* determine the directory associated with the clicked button */
   directory = thunar_location_button_get_file (button);
 
+  /* open in tab */
+  if (open_in_tab)
+    {
+      thunar_navigator_open_new_tab (THUNAR_NAVIGATOR (buttons), directory);
+      return;
+    }
+
   /* disconnect from previous current directory (if any) */
   if (G_LIKELY (buttons->current_directory != NULL))
     g_object_unref (G_OBJECT (buttons->current_directory));
@@ -1218,18 +1236,23 @@ thunar_location_buttons_context_menu (ThunarLocationButton  *button,
           /* setup the "Open" action */
           action = gtk_action_group_get_action (buttons->action_group, "location-buttons-open");
           thunar_gtk_action_set_tooltip (action, _("Open \"%s\" in this window"), display_name);
-          g_object_set_data_full (G_OBJECT (action), I_("thunar-file"), g_object_ref (G_OBJECT (file)), (GDestroyNotify) g_object_unref);
+          g_object_set_qdata_full (G_OBJECT (action), thunar_file_quark, g_object_ref (G_OBJECT (file)), (GDestroyNotify) g_object_unref);
           gtk_action_set_sensitive (action, (file != buttons->current_directory));
 
           /* setup the "Open in New Window" action */
           action = gtk_action_group_get_action (buttons->action_group, "location-buttons-open-in-new-window");
           thunar_gtk_action_set_tooltip (action, _("Open \"%s\" in a new window"), display_name);
-          g_object_set_data_full (G_OBJECT (action), I_("thunar-file"), g_object_ref (G_OBJECT (file)), (GDestroyNotify) g_object_unref);
+          g_object_set_qdata_full (G_OBJECT (action), thunar_file_quark, g_object_ref (G_OBJECT (file)), (GDestroyNotify) g_object_unref);
+
+          /* setup the "Open in New Tab" action */
+          action = gtk_action_group_get_action (buttons->action_group, "location-buttons-open-in-new-tab");
+          thunar_gtk_action_set_tooltip (action, _("Open \"%s\" in a new tab"), display_name);
+          g_object_set_qdata_full (G_OBJECT (action), thunar_file_quark, g_object_ref (G_OBJECT (file)), (GDestroyNotify) g_object_unref);
 
           /* setup the "Create Folder..." action */
           action = gtk_action_group_get_action (buttons->action_group, "location-buttons-create-folder");
           thunar_gtk_action_set_tooltip (action, _("Create a new folder in \"%s\""), display_name);
-          g_object_set_data_full (G_OBJECT (action), I_("thunar-file"), g_object_ref (G_OBJECT (file)), (GDestroyNotify) g_object_unref);
+          g_object_set_qdata_full (G_OBJECT (action), thunar_file_quark, g_object_ref (G_OBJECT (file)), (GDestroyNotify) g_object_unref);
           gtk_action_set_sensitive (action, thunar_file_is_writable (file));
           gtk_action_set_visible (action, !thunar_file_is_trashed (file));
 
@@ -1241,13 +1264,13 @@ thunar_location_buttons_context_menu (ThunarLocationButton  *button,
           /* setup the "Paste Into Folder" action */
           action = gtk_action_group_get_action (buttons->action_group, "location-buttons-paste-into-folder");
           thunar_gtk_action_set_tooltip (action, _("Move or copy files previously selected by a Cut or Copy command into \"%s\""), display_name);
-          g_object_set_data_full (G_OBJECT (action), I_("thunar-file"), g_object_ref (G_OBJECT (file)), (GDestroyNotify) g_object_unref);
+          g_object_set_qdata_full (G_OBJECT (action), thunar_file_quark, g_object_ref (G_OBJECT (file)), (GDestroyNotify) g_object_unref);
           gtk_action_set_sensitive (action, thunar_clipboard_manager_get_can_paste (clipboard));
 
           /* setup the "Properties..." action */
           action = gtk_action_group_get_action (buttons->action_group, "location-buttons-properties");
           thunar_gtk_action_set_tooltip (action, _("View the properties of the folder \"%s\""), display_name);
-          g_object_set_data_full (G_OBJECT (action), I_("thunar-file"), g_object_ref (G_OBJECT (file)), (GDestroyNotify) g_object_unref);
+          g_object_set_qdata_full (G_OBJECT (action), thunar_file_quark, g_object_ref (G_OBJECT (file)), (GDestroyNotify) g_object_unref);
 
           /* run the menu on the screen on the buttons' screen */
           menu = gtk_ui_manager_get_widget (buttons->ui_manager, "/location-buttons-context-menu");
@@ -1275,7 +1298,7 @@ thunar_location_buttons_action_create_folder (GtkAction             *action,
   _thunar_return_if_fail (GTK_IS_ACTION (action));
 
   /* determine the directory for the action */
-  directory = g_object_get_data (G_OBJECT (action), "thunar-file");
+  directory = g_object_get_qdata (G_OBJECT (action), thunar_file_quark);
   if (G_UNLIKELY (directory == NULL))
     return;
 
@@ -1358,7 +1381,7 @@ thunar_location_buttons_action_open (GtkAction             *action,
   _thunar_return_if_fail (GTK_IS_ACTION (action));
 
   /* determine the directory for the action */
-  directory = g_object_get_data (G_OBJECT (action), "thunar-file");
+  directory = g_object_get_qdata (G_OBJECT (action), thunar_file_quark);
   if (G_LIKELY (directory != NULL && thunar_file_is_directory (directory)))
     {
       /* open the folder in this window */
@@ -1368,6 +1391,26 @@ thunar_location_buttons_action_open (GtkAction             *action,
 
 
 
+static void
+thunar_location_buttons_action_open_in_new_tab (GtkAction             *action,
+                                                ThunarLocationButtons *buttons)
+{
+  ThunarFile *directory;
+
+  _thunar_return_if_fail (THUNAR_IS_LOCATION_BUTTONS (buttons));
+  _thunar_return_if_fail (GTK_IS_ACTION (action));
+
+  /* determine the directory for the action */
+  directory = g_object_get_qdata (G_OBJECT (action), thunar_file_quark);
+  if (G_LIKELY (directory != NULL))
+    {
+      /* open tab in thsi window */
+      thunar_navigator_open_new_tab (THUNAR_NAVIGATOR (buttons), directory);
+    }
+}
+
+
+
 static void
 thunar_location_buttons_action_open_in_new_window (GtkAction             *action,
                                                    ThunarLocationButtons *buttons)
@@ -1379,7 +1422,7 @@ thunar_location_buttons_action_open_in_new_window (GtkAction             *action
   _thunar_return_if_fail (GTK_IS_ACTION (action));
 
   /* determine the directory for the action */
-  directory = g_object_get_data (G_OBJECT (action), "thunar-file");
+  directory = g_object_get_qdata (G_OBJECT (action), thunar_file_quark);
   if (G_LIKELY (directory != NULL))
     {
       /* open a new window for the directory */
@@ -1402,7 +1445,7 @@ thunar_location_buttons_action_paste_into_folder (GtkAction             *action,
   _thunar_return_if_fail (GTK_IS_ACTION (action));
 
   /* determine the directory for the action */
-  directory = g_object_get_data (G_OBJECT (action), "thunar-file");
+  directory = g_object_get_qdata (G_OBJECT (action), thunar_file_quark);
   if (G_LIKELY (directory != NULL))
     {
       /* paste files from the clipboard to the folder represented by this button */
@@ -1426,7 +1469,7 @@ thunar_location_buttons_action_properties (GtkAction             *action,
   _thunar_return_if_fail (GTK_IS_ACTION (action));
 
   /* determine the directory for the action */
-  directory = g_object_get_data (G_OBJECT (action), "thunar-file");
+  directory = g_object_get_qdata (G_OBJECT (action), thunar_file_quark);
   if (G_LIKELY (directory != NULL))
     {
       /* determine the toplevel window */
